// Author: Hugo Pointcheval
// Email: git@pcl.ovh
// -----
// File: sign_in_form.dart
// Created Date: 19/08/2022 15:24:37
// Last Modified: Wed Nov 16 2022
// -----
// Copyright (c) 2022

import 'package:flutter/material.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class _EmailInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<SignInCubit<int>>(
      field: AuthFormField.email,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (email) => cubit.emailChanged(email),
          keyboardType: TextInputType.emailAddress,
          decoration: InputDecoration(
            labelText: 'Email',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid email' : null,
          ),
        );
      }),
    );
  }
}

class _PasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<SignInCubit<int>>(
      field: AuthFormField.password,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (pwd) => cubit.passwordChanged(pwd),
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'Password',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid password' : null,
          ),
        );
      }),
    );
  }
}

class _SignInButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<SignInCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated ? () => cubit.signInWithEmailAndPassword() : null,
                child: const Text('Sign in with credentials'),
              );
      }),
    );
  }
}

class _SignInAnonymouslyButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<SignInCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: () => cubit.signInAnonymously(),
                child: const Text('Sign in anonymously'),
              );
      }),
    );
  }
}

class SignInForm extends StatelessWidget {
  const SignInForm({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SignInListener<int>(
      onError: (context, status, errorMessage) => ScaffoldMessenger.of(context)
        ..hideCurrentSnackBar()
        ..showSnackBar(
          SnackBar(content: Text(errorMessage ?? 'Sign In Failure')),
        ),
      child: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            _EmailInput(),
            const SizedBox(height: 8),
            _PasswordInput(),
            const SizedBox(height: 16),
            _SignInButton(),
            const SizedBox(height: 16),
            _SignInAnonymouslyButton(),
          ],
        ),
      ),
    );
  }
}
